@@ -38,7 +38,6 @@
   #:use-module (nyacc lang arch-info)
   #:use-module (nyacc lang c99 cpp)
   #:use-module (nyacc lang c99 parser)
-  #:use-module (nyacc lang c99 munge)
   #:use-module (nyacc lang c99 munge-base)
   #:use-module (rnrs arithmetic bitwise)
   #:use-module (system foreign)
@@ -97,7 +96,10 @@
 (define incr-size cx-incr-size)
 (define maxi-size cx-maxi-size)
 
-(define (sizeof-mtail mtail)
+(define (mkcdl specl declrs)
+	  (map (lambda (declr) `(comp-declr-list ,specl ,declr)) declrs))
+
+(define (sizeof-mtail mtail udict)
 
   (define (exec/decl decl size base-align update)
     (let ((mtail (sx-tail (sx-find 'type-spec (sx-ref decl 1))))
@@ -108,7 +110,7 @@
 	    (call-with-values
 		(lambda ()
 		  (sizeof-mtail
-		   (cdr (m-unwrap-declr (car declrs) mtail))))
+		   (cdr (m-unwrap-declr (car declrs) mtail)) udict))
 	      (lambda (elt-size elt-align)
 		(loop (update elt-size elt-align size)
 		      (max elt-align base-align) (cdr declrs))))))))
@@ -121,9 +123,9 @@
     (`((float-type ,name))
      (values (sizeof-basetype name) (alignof-basetype name)))
     (`((array-of ,size) . ,rest)
-     (let ((mult (eval-c99-cx size)))
+     (let ((mult (eval-c99-cx size udict)))
        (call-with-values
-	   (lambda () (sizeof-mtail rest))
+	   (lambda () (sizeof-mtail rest udict))
 	 (lambda (size align)
 	   (values (* mult size) align)))))
     (`((struct-def (field-list . ,fields)))
@@ -132,7 +134,6 @@
 	((null? flds) (values (incr-size 0 align size) align))
 	((eq? 'comp-decl (sx-tag (car flds)))
 	 (call-with-values
-	     ;;(lambda () (incr/decl (car flds) size align))
 	     (lambda () (exec/decl (car flds) size align incr-size))
 	   (lambda (size align)
 	     (loop size align (cdr flds)))))
@@ -156,7 +157,7 @@
   (let* ((udecl `(udecl ,specl ,declr))
 	 (xdecl (expand-typerefs udecl udict))
 	 (mdecl (udecl->mdecl xdecl)))
-    (sizeof-mtail (cdr mdecl))))
+    (sizeof-mtail (cdr mdecl) udict)))
 
 (define (trim-mtail mtail)
   (case (caar mtail)
@@ -190,10 +191,10 @@
       (lambda () (size-and-align-of-type tree udict))
     (lambda (size align) align)))
 
-;; @deffn {Procedure} size-or-align-of-expr tree [udict]
+;; @deffn {Procedure} size-and-align-of-expr tree [udict]
 ;; => (values sizeof-val align-of)
 ;; @end deffn
-(define* (size-or-align-of-expr tree #:optional (udict '()))
+(define* (size-and-align-of-expr tree #:optional (udict '()))
 
   (define (sizeof-literal tree)
     (sx-match tree
@@ -230,20 +231,20 @@
 
   (or
    (sizeof-literal tree)
-   (sizeof-mtail (gen-mtail tree))))
+   (sizeof-mtail (gen-mtail tree) udict)))
 
 (define* (eval-sizeof-expr tree #:optional (udict '()))
   (call-with-values
-      (lambda () (size-or-align-of-expr tree udict))
+      (lambda () (size-and-align-of-expr tree udict))
     (lambda (size align) size)))
 
 
 ;; =============================================================================
 
-(define* (gen-offsets mtail #:optional (base 0))
+(define (gen-offsets mtail base udict)
 
   (define (mt-al mtail)
-    (call-with-values (lambda () (sizeof-mtail mtail))
+    (call-with-values (lambda () (sizeof-mtail mtail udict))
       (lambda (sz al) al)))
 
   (define (do-aggr flds update)
@@ -253,7 +254,7 @@
 	 ((pair? decls)
 	  (let* ((mdecl (udecl->mdecl (car decls)))
 		 (name (car mdecl)) (mtail (cdr mdecl)))
-	    (call-with-values (lambda () (gen-offsets mtail (+ base siz)))
+	    (call-with-values (lambda () (gen-offsets mtail (+ base siz) udict))
 	      (lambda (el-sz el-al el-os)
 		;;(sferr "sizeof(~S)=~S\n" name el-sz)
 		(let ((oval (if (pair? el-os) el-os
@@ -261,10 +262,13 @@
 		  (loop (update el-sz el-al siz) (max aln el-al)
 			(acons name oval offs) (cdr decls) flds))))))
 	 ((pair? flds)
-	  (if (memq (sx-tag (car flds)) '(comp-decl comp-udecl))
-	      (loop siz aln offs
-		    (map cdr (dictize-comp-decl (car flds))) (cdr flds))
-	      (loop siz aln offs decls (cdr flds))))
+	  (sx-match (car flds)
+	    ((comp-decl ,specl (comp-declr-list . ,declrs))
+	     (loop siz aln offs (mkcdl specl declrs) (cdr flds)))
+	    ((comp-udecl ,specl ,declr)
+	     (loop siz aln offs (list declr) (cdr flds)))
+	    (,_
+	     (loop siz aln offs decls (cdr flds)))))
 	 (else (values siz aln (reverse offs)))))))
 
   (match mtail
@@ -278,10 +282,10 @@
      (let ((sz (sizeof-basetype name)) (al (alignof-basetype name)))
        (values sz al (incr-size 0 al base))))
     (`((array-of ,dim) . ,rest)
-     (call-with-values (lambda () (gen-offsets rest base))
+     (call-with-values (lambda () (gen-offsets rest base udict))
        (lambda (el-sz el-al el-of)
 	 (let ((base (incr-size 0 el-al base)))
-	   (let ((dim (eval-c99-cx dim)))
+	   (let ((dim (eval-c99-cx dim udict)))
 	     (values (* dim el-sz) el-al (cons (cons dim el-sz) el-of)))))))
     (`((struct-def (field-list . ,flds)) . ,rest)
      (do-aggr flds incr-size))
@@ -292,43 +296,22 @@
     (_ (sferr "c99/gen-offsets: missed\n") (pperr mtail)
        (throw 'c99-error "coding error"))))
 
-(define (find-offsets mtail)
+(define (find-offsets mtail udict)
   (call-with-values
-      (lambda () (gen-offsets mtail))
+      (lambda () (gen-offsets mtail 0 udict))
     (lambda (size align offsets)
       offsets)))
   
-;; @deffn {Procedure} offsetof-mtail mtail desig [base] => offset alignment
+;; @deffn {Procedure} offsetof-mtail mtail desig base udict => offset alignment
 ;; @end deffn
-(define* (offsetof-mtail mtail desig #:optional (base 0))
+(define (offsetof-mtail mtail desig base udict)
 
-  (define (exec/decl decl size align update)
-    ;; => values size align offs
-    (let ((mtail (sx-tail (sx-find 'type-spec (sx-ref decl 1))))
-	  (declrs (or (and=> (sx-ref decl 2) sx-tail) '((ident "_")))))
-      (let loop ((size size) (align align) (declrs declrs))
-	(cond
-	 ((null? declrs) (values size align))
-	 ((null? desig) (error "failed"))
-	 ((equal? (declr-ident (car declrs)) (car desig))
-	  (values (incr-size 0 align size) #f))
-	 (else
-	  (call-with-values
-	      (lambda ()
-		(let ((mdecl (m-unwrap-declr (car declrs) mtail)))
-		  (offsetof-mtail (cdr mdecl) desig base)))
-	    (lambda (elt-size elt-align)
-	      (loop (update elt-size elt-align size)
-		    (max elt-align align)
-		    (cdr declrs)))))))))
-
-  ;;(sferr "offsetof-mtail desig=~S  base=~S\n" desig base)
   (match mtail
 
-    (`((array-of ,size) . ,rest)
-     (let ((mult (eval-c99-cx size)))
+    (`((array-of ,dim) . ,rest)
+     (let ((mult (eval-c99-cx dim udict)))
        (call-with-values
-	   (lambda () (offsetof-mtail rest base))
+	   (lambda () (offsetof-mtail rest (cdr desig) base udict))
 	 (lambda (size align offs) offs))))
     
     (`((struct-def (field-list . ,fields)))
@@ -342,21 +325,23 @@
 		(name (car mdecl))
 		(mtail (cdr mdecl)))
 	   (call-with-values
-	       (lambda () (sizeof-mtail mtail))
+	       (lambda () (sizeof-mtail mtail udict))
 	     (lambda (elt-sz elt-al)
 	       (cond
 		((string=? name dsg)
 		 (let* ((xoffs (incr-size 0 elt-al offs)))
 		   (if (null? (cdr desig)) xoffs
-		       (offsetof-mtail mtail (cdr desig) xoffs))))
+		       (offsetof-mtail mtail (cdr desig) xoffs udict))))
 		(else
 		 (loop (incr-size elt-sz elt-al offs) (max elt-al aln)
 		       dsg (cdr decls) flds)))))))
 	((pair? flds)
-	 (if (memq (sx-tag (car flds)) '(comp-decl comp-udecl))
-	     (loop offs aln dsg
-		   (map cdr (dictize-comp-decl (car flds))) (cdr flds))
-	     (loop offs aln dsg decls (cdr flds))))
+	  (sx-match (car flds)
+	    ((comp-decl ,specl (comp-declr-list . ,declrs))
+	     (loop offs aln dsg (mkcdl specl declrs) (cdr flds)))
+	    ((comp-udecl ,specl ,declr)
+	     (loop offs aln dsg (mkcdl specl (list declr)) (cdr flds)))
+	    (,_ (loop offs aln dsg decls (cdr flds)))))
 	(else (values offs aln)))))
 	   
     (`((union-def (field-list . ,fields)))
@@ -370,21 +355,23 @@
 		(name (car mdecl))
 		(mtail (cdr mdecl)))
 	   (call-with-values
-	       (lambda () (sizeof-mtail mtail))
+	       (lambda () (sizeof-mtail mtail udict))
 	     (lambda (elt-sz elt-al)
 	       (cond
 		((string=? name dsg)
 		 (let* ((xoffs (maxi-size 0 elt-al offs)))
 		   (if (null? (cdr desig)) xoffs
-		       (offsetof-mtail mtail (cdr desig) xoffs))))
+		       (offsetof-mtail mtail (cdr desig) xoffs udict))))
 		(else
 		 (loop (maxi-size 0 elt-al offs) (max elt-al aln)
 		       dsg (cdr decls) flds)))))))
 	((pair? flds)
-	 (if (memq (sx-tag (car flds)) '(comp-decl comp-udecl))
-	     (loop offs aln dsg
-		   (map cdr (dictize-comp-decl (car flds))) (cdr flds))
-	     (loop offs aln dsg decls (cdr flds))))
+	  (sx-match (car flds)
+	    ((comp-decl ,specl (comp-declr-list . ,declrs))
+	     (loop offs aln dsg (mkcdl specl declrs) (cdr flds)))
+	    ((comp-udecl ,specl ,declr)
+	     (loop offs aln dsg (mkcdl specl (list declr)) (cdr flds)))
+	    (,_ (loop offs aln dsg decls (cdr flds)))))
 	(else (values offs aln)))))
 
     (_ (sferr "c99/eval-sizeof-mtail: missed\n") (pperr mtail)
@@ -420,7 +407,7 @@
 	    (xdecl (expand-typerefs udecl udict))
 	    (mdecl (udecl->mdecl xdecl))
 	    (desig (unwrap-designator expr udict)))
-       (offsetof-mtail (cdr mdecl) desig)))
+       (offsetof-mtail (cdr mdecl) desig udict 0)))
     (,_ #f)))
 
 
@@ -479,6 +466,7 @@
 	    ((neg) (uop - (ev1 tree)))
 	    ((not) (and tree (if (equal? 0 (ev1 tree)) 1 0)))
 
+	    ((mul) (bop * (ev1 tree) (ev2 tree)))
 	    ((div) (bop / (ev1 tree) (ev2 tree)))
 	    ((mod) (bop modulo (ev1 tree) (ev2 tree)))
 	    ((add) (bop + (ev1 tree) (ev2 tree)))
