@@ -203,7 +203,6 @@
 ;; Given declaration return a list of new typenames (via @code{typedef}).
 ;; @end deffn
 (define (find-new-typenames decl)
-  ;; like declr-id in util2.scm
   (define (declr->id-name declr)
     (case (car declr)
       ((ident) (sx-ref declr 1))
@@ -269,9 +268,14 @@
 	 ((eq? ch #\newline) (unread-char ch) (reverse-list->string cl))
 	 ((eq? ch #\\)
 	  (let ((c2 (read-char)))
-	    (if (eq? c2 #\newline)
-		(loop cl (read-char))
-		(loop (cons* c2 ch cl) (read-char)))))
+	    (if (eq? c2 #\return)
+		(let ((c3 (read-char)))
+		  (if (eq? c3 #\newline)
+		      (loop cl (read-char))
+		      (loop (cons* c3 c2 ch cl) (read-char))))
+		(if (eq? c2 #\newline)
+		    (loop cl (read-char))
+		    (loop (cons* c2 ch cl) (read-char))))))
 	 ((eq? ch #\/) ;; swallow comments, even w/ newlines
 	  (let ((c2 (read-char)))
 	    (cond
